% function that generates guide with level sets method for ipol demo
%
% Version 0.0, august 2017
% Author lara.raad@gmail.com
%
% level sets method
% Creates random guide planks using level sets of a smooth function with
% up to two Gaussian bumps.
%
% Version 0.2, May 2017
% Author jlezama@fing.edu.uy

function [s1,s3,s4,s5] = generate_guide_ipol(params)

% params = [H W sigma_blob sigma_disp max_disp blob_vert_pos...
% blob_hor_pos num_int blob_mult dilat_vert dilat_hor surf_decay...
% vert_disp_mult]

% check number of input parameters
errormsg = ['there should be 23 specified parameters  -- '...
'params = [H W sigma_blob sigma_disp max_disp blob_vert_pos '...
'blob_hor_pos num_int blob_mult dilat_vert dilat_hor surf_decay'...
'vert_disp_mult]'];
assert(length(params)==23,errormsg);

% get parameters
H = params(1); % height
W = params(2); % width
sigma_blob = params(3:4);
sigma_displacement = params(5);
max_displacement = params(6);
blob_vertical_position =  params(7:8);
blob_horizontal_position = params(9:10);
num_intervals = params(11);
blob_multiplier = params(12:13);
dilation_length_vertical = params(14);
dilation_length_horizontal = params(15);
surface_decay = params(16);
vertical_displacement_multiplier = params(17);
sig1_11 = params(18);%30;
sig1_22 = params(19);%7;
sig1_12 = params(20);%2;
sig2_11 = params(21);%2;
sig2_22 = params(22);%70;
sig2_12 = params(23);%70;


% initalize surface
[X, Y] = meshgrid(1:W,1:H);
initial_surface = (1-normalize(abs(X-W/2).^surface_decay));
s1 = initial_surface;
s1 = s1(round(H/20)+1:end-round(H/20),...
                          round(W/20)+1:end-round(W/20),:);

% add two gaussian bumps
disp('generating gaussian blob 1')
g1 = gaussian_blob(H,W, round(H*blob_vertical_position(1)),...
    round(W*blob_horizontal_position(1)),...
    [sig1_11 sig1_12;sig1_12 sig1_22]*sigma_blob(1));

disp('generating gaussian blob 2')
g2 = gaussian_blob(H,W, round(H*blob_vertical_position(2)),...
    round(W*blob_horizontal_position(2)),...
    [sig2_11 sig2_12;sig2_12 sig2_22]*sigma_blob(2));

mysurface = initial_surface + g1*blob_multiplier(1) + g2*blob_multiplier(2);
mysurface = normalize(mysurface);

% add some random displacement
disp('generating random displacement')
[vx, vy] = random_displacement(H, W, sigma_displacement, max_displacement, 0, 0 ,0);

[X, Y] = meshgrid(1:W,1:H);
mysurface = interp2(mysurface, X + vx , Y + vy*vertical_displacement_multiplier); % stronger on vertical
mysurface = normalize(mysurface);
mysurface(isnan(mysurface))=0;
s3 = mysurface;
s3 = s3(round(H/20)+1:end-round(H/20),...
                          round(W/20)+1:end-round(W/20),:);

% compute Level Lines
intervals = [0:1./num_intervals:1];
intervals = intervals.^2; % stretch intervals to have larger gaps in cathedrals
intervals = sort(normalize(intervals + randn(size(intervals))/300));
L = length(intervals);

% colorize
red = zeros(H,W);
green = zeros(H,W);
blue = zeros(H,W);

light = [179, 150, 108
    158, 133, 93
    186, 158, 110
    167, 136, 92
    175, 150, 109
    211, 181, 131]/255;

dark = [110, 78, 29
    114, 81, 36
    112, 82, 32
    107, 75, 18
    132, 96, 46
    139, 110, 66]/255;

assert(size(dark,1)==size(light,1))
Ncolors = size(dark,1);

disp('painting intervals')
for i=1:L-1
    z = find(mysurface>=intervals(i) & mysurface<=intervals(i+1));
    
    if mod(i,2)
        colors = dark;
    else
        colors = light;
    end
    
    c = randi(Ncolors);
    red(z) = colors(c,1);
    green(z) = colors(c,2);
    blue(z) = colors(c,3);
end

new_colored = cat(3,red,green,blue);

% cut borders
new_colored = new_colored(round(H/20)+1:end-round(H/20),...
                          round(W/20)+1:end-round(W/20),:);
s4 = new_colored;                      

disp('postprocessing gradients')
new_postprocessed = gradient_to_guide(normalize(imgradient(rgb2gray(new_colored))), dilation_length_vertical, dilation_length_horizontal, dark, light);
s5 = new_postprocessed;